﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public abstract class CandidatesGenerator
	{
		protected int size;
		protected UndirectedGraph graph;
		public static bool smartStart = true;

		public virtual ThreadSafeQueue<Community> generate(UndirectedGraph graph, BinaryMatrix dm)
		{
			this.graph = graph;
			size = graph.Count;

			ThreadSafeQueue<Community> queue = generate(new ThreadSafeQueue<Community>());
			int qSize = queue.Count;
			if(smartStart && dm!=null)
			{
				while (qSize > 1900)
				{
					Community c = queue.Dequeue();
					if (c.Size == 1)
					{
						int v = c.getVertices().Keys.ElementAt(0);
						BitArray row = dm.getRow(v);
						double d = double.MaxValue;
						int best = -1;
						foreach (int u in c.getNeighbours().Keys)
						{
							BitArray rowOther = dm.getRow(u);
							double jd = row.JaccardDistance(rowOther);
							if (jd < d)
							{
								d = jd;
								best = u;
							}
						}
						if (best != -1)
						{
							c.AddVertice(best);
						}
					}
					queue.Enqueue(c);
					qSize--;
				}
			}
			return queue;
		}

		protected abstract ThreadSafeQueue<Community> generate(ThreadSafeQueue<Community> queue);

		public override string ToString()
		{
			string name = this.GetType().Name;
			return name.Substring(name.LastIndexOf('.') + 1)+(smartStart?" with":" without")+" smart start";
		}

		public abstract string abbr();
	}
}
