﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	/**
	 * Create a candidate set with a minimum shortest path distance between to candidates
	 */
	class SharedNeighboursCG : CandidatesGenerator
	{
		private int distance;
		private int p;

		public SharedNeighboursCG(int distance)
		{
			this.distance = distance;
		}

		protected override ThreadSafeQueue<Community> generate(ThreadSafeQueue<Community> queue)
		{
			HashSet<int> seen = new HashSet<int>();
			for (int i = 0; i < graph.Count; i++)
			{
				if (seen.Contains(i))
					continue;

				Community c = new Community(graph);
				c.AddVertice(i);
				seen.Add(i);

				HashSet<int> neighbours = new HashSet<int>();
				neighbours.Add(i);
				for (int d = 0; d < distance; d++)
				{
					List<int> add = new List<int>();
					foreach (int v in neighbours)
					{
						foreach (int u in graph.getNeighbours(v))
							add.Add(u);
					}
					foreach (int v in add)
						neighbours.Add(v);
				}
				foreach (int v in neighbours)
					seen.Add(v);
				queue.Enqueue(c);
			}
			return queue;
		}

		public override string ToString()
		{
			string name = this.GetType().Name;
			return name.Substring(name.LastIndexOf('.') + 1) +"("+distance+")"+ (smartStart ? " with" : " without") + " smart start";
		}

		public override string abbr()
		{
			return "sn"+distance+"-" + (smartStart ? "y" : "n");
		}
	}
}
