﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	public class BeamPattern : IComparable<BeamPattern>, IEquatable<BeamPattern>
	{
		public HashSet<int> pattern;
		public double gain;
		public List<int> rows;

		public int CompareTo(BeamPattern other)
		{
			if(other.gain < gain)
				return -1;
			else if(other.gain > gain)
				return 1;

			if (other.pattern.Count > pattern.Count)
				return -1;
			else if (other.pattern.Count < pattern.Count)
				return 1;
			return 0;
		}

		public void AddRow(int r)
		{
			rows.Add(r);
		}

		public void AddAttribute(int a)
		{
			pattern.Add(a);
		}

		public BeamPattern()
		{
			pattern = new HashSet<int>();
			rows = new List<int>();
		}

		public BeamPattern(BeamPattern p)
		{
			pattern = new HashSet<int>(p.pattern);
			rows = new List<int>(p.rows);
		}

		public override string ToString()
		{
			List<int> l = new List<int>(pattern);
			l.Sort();
			return "{"+Tools.listToString(l)+"}";
		}

		public bool Equals(BeamPattern other)
		{
			if (pattern.Count != other.pattern.Count)
				return false;
			return pattern.IsSubsetOf(other.pattern) && other.pattern.IsSubsetOf(pattern);
		}

		public override int GetHashCode()
		{
			int hash = pattern.Count*pattern.Count;
			foreach (int i in pattern)
				hash = hash ^ i;
			return hash;

		}
	}
}
