﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	class RemineBuilder : Builder
	{
		/**
		 * Just store data and frequency
		 */
		public RemineBuilder(BinaryMatrix data) : base(data)
		{
		}

		/**
		 * Build a classifier given the set of rows with a classlabel 1
		 */
		public override Classifier build(HashSet<int> trueRows)
		{
			//create first partition with all rows
			List<int> rows = new List<int>(data.M);
			for (int i = 0; i < data.M; i++)
			{
				rows.Add(i);
			}
			Partition all = new Partition(data, trueRows, rows);
			List<Partition> partitioning = new List<Partition>();
			partitioning.Add(all);

			//create classifier
			RemineClassifier classifier = new RemineClassifier();
			all.splitNode = classifier.root;

			//start partitioning
			int max = 10; //maximum number of iterations
			while (max > 0)
			{
				Console.WriteLine("it " + max);
				//find best pattern in each block
				List<Pattern> newPatterns = new List<Pattern>();
				foreach (Partition p in partitioning)
				{
					Pattern pattern = new Pattern(p.findBestPattern().pattern);
					if (pattern != null && pattern.items != null)
					{
						newPatterns.Add(pattern); //save new patterns
					}
				}
				//split all partitions according to the new patterns
				partitioning = split(partitioning, newPatterns, classifier);
				max--;
			}

			foreach (Partition p in partitioning)
			{
				int t=0,f=0;
				foreach (int r in p.partitionRows)
				{
					if (trueRows.Contains(r))
						t++;
					else
						f++;
				}
				p.splitNode.classlabel = t >= f;
			}

			return classifier;

		}



		/**
		 * Create an FP Tree with the algorithm described in "Mining frequent patterns without candidata generation: A frequent pattern tree approach"
		 * works very well
		 */



		public List<Partition> split(List<Partition> partitioning, List<Pattern> newPatterns, RemineClassifier classifier)
		{
			List<Partition> newParitioning = new List<Partition>();

			//loop trough current partitions
			foreach (Partition p in partitioning)
			{
				if (p.Count <= 1)
					continue;
				//split each partition on all new patterns
				Queue<Partition> splitQueue = new Queue<Partition>();
				splitQueue.Enqueue(p);
				
				foreach (Pattern pattern in newPatterns)
				{
					//look at number of partitions before starting splitting on this pattern
					int count = splitQueue.Count;
					while (count > 0) //split only these ones, and add result to use for next pattern
					{
						Partition sp = splitQueue.Dequeue();
						List<Partition> splitresult = sp.split(pattern);
						foreach(Partition sr in splitresult)
						{
							splitQueue.Enqueue(sr);
						}
						count--;
					}
				}

				//all patterns done for this partition, add to new partitioning
				while (splitQueue.Count > 0)
				{
					newParitioning.Add(splitQueue.Dequeue());
				}
			}
			return newParitioning;
		}
	}
}
