﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public class BinaryMatrixOud : BinaryMatrix
    {
        protected int[][] data;

        public BinaryMatrixOud(int m, int n)
        {
            this.m = m;
            this.n = n;
            int size = (n / 32) + ((n % 32 == 0)?0:1);
            data = new int[m][];
			for (int i = 0; i < m; i++)
				data[i] = new int[size];
        }

		public BinaryMatrixOud(string file)
		{
			StreamReader reader = new StreamReader(file);
			string line = null;
			while (line == null || line[0] == '#' || line[0] == '%')
				line = reader.ReadLine();
			m = int.Parse(line);
			n = int.Parse(reader.ReadLine());
			data = new int[int.Parse(reader.ReadLine())][];
			int size = int.Parse(reader.ReadLine());
			for (int j = 0; j < m; j++)
				data[j] = new int[size];

			int type = int.Parse(reader.ReadLine());
			int i=0;
			while(null != (line = reader.ReadLine()))
			{
				string[] words=line.Split(' ');
				for(int j=0; j<size; j++)
				{
					data[i][j] = Convert.ToInt32(words[j], type);
				}
				i++;
			}
			reader.Close();
		}


		public override BitArray getRow(int i)
		{
			return new BitArray(data[i], n);
		}


		public override bool get(int i, int j)
        {
            return (1 << (31 -(j % 32)) & data[i][ j / 32]) != 0;
        }

		public override void set(int i, int j, bool v)
		{
			if (v)
				data[i][j / 32] = (1 << (31 - (j % 32))) | data[i][j / 32];
			else
				data[i][j / 32] = ~(1 << (31 - (j % 32))) & data[i][j / 32];
		}

		public override List<int> getIndexesByValuePerRow(int i,bool v)
		{
			List<int> indexes = new List<int>();
			if (v)
			{
				for (int j = 0; j < data[0].Length; j++)
				{
					if (data[i][j] != 0)
					{
						int min = Math.Min((j + 1) * 32,n);
						for (int k = j * 32; k < min; k++)
						{
							if (get(i,k))
								indexes.Add(k);
						}
					}
				}
			}
			else
			{
				for (int j = 0; j < data[0].Length; j++)
				{
			
					int min = Math.Min((j + 1) * 32,n);
					for (int k = j * 32; k < min; k++)
					{
						if (data[i][j] == 0 || (data[i][j] != 0 && !get(i,k)))
						{
							indexes.Add(k);
						}
					}
				}
			}
			return indexes;
		}

		public override int countRow(int i, bool v)
		{
			if (!v)
				return n - countRow(i, true);
			int count = 0;
			for (int j = 0; j < data[0].Length; j++)
			{
				if (data[i][j] != 0)
				{
					for (int k = j * 32; k < (j + 1) * 32; k++)
					{
						if (get(i, k))
							count++;
					}
				}
			}
			return count;
		}



		public override int[] columnSum()
		{
			int[] sum = new int[n];
			for (int i = 0; i < m; i++)
			{
				for (int j = 0; j < data[0].Length; j++)
				{
					if (data[i][j] != 0)
					{
						int min = Math.Min(n, (j + 1) * 32);
						for (int k = j * 32; k < min; k++)
						{
							if (get(i, k))
							{
								sum[k]++;
							}
						}
					}
				}
			}
			return sum;
		}
		
		public int count(bool v)
		{
			int count=0;
			for (int i = 0; i < m; i++)
			{
				count += countRow(i, v);
			}
			return count;
		}

		public override int countValue(bool v)
		{
			return count(v);
		}

		protected override void print(TextWriter writer, int type)
		{
			int b = (type == 16) ? 16 : (type == 10 ? 10 : 2);

			//info
			writer.WriteLine("##### Matrix file ######");
			writer.WriteLine("# Size:");
			writer.WriteLine("#     m: " + m);
			writer.WriteLine("#     n: " + n);
			writer.WriteLine("# Encoding:");
			writer.WriteLine("#  base: " + b);
			writer.WriteLine("###### data ######");

			//parse info
			writer.WriteLine(m);
			writer.WriteLine(n);
			writer.WriteLine(data.Length);
			writer.WriteLine(data[0].Length);
			writer.WriteLine(b);


			//data
			for (int i = 0; i < data.Length; i++)
			{
				for (int j = 0; j < data[0].Length; j++)
				{
					if (j < data[0].Length - 1)
						writer.Write(format(data[i][j], b) + " ");
					else
					{
						if (i < data.Length - 1)
							writer.WriteLine(format(data[i][j], b));
						else
							writer.Write(format(data[i][j], b));
					}
				}
			}
		}
    }
}
