﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public abstract class UndirectedGraph
	{
		protected int size;
		protected int numEdges;
		protected int[] degreeTable;
		protected bool degreeTableChanged;

		//public UndirectedGraph()
		//{
		//}

		public UndirectedGraph(int size)
		{
			this.size = size;
			init(size);
			degreeTable = new int[size];
			degreeTableChanged = true;
		}

		public UndirectedGraph(string path)
		{
			StreamReader reader = new StreamReader(path);
			size = int.Parse(reader.ReadLine());
			init(size);
			degreeTable = new int[size];
			degreeTableChanged = true;
			string line;
			while(null != (line = reader.ReadLine()))
			{
				string[] words = line.Split(' ');
				int id = int.Parse(words[0]);
				for (int i = 1; i < words.Count(); i++)
				{
					addEgde(id, int.Parse(words[i]));
				}
			}
			reader.Close();
		}

		abstract protected void init(int size);

		public void write(string path)
		{
			StreamWriter writer = new StreamWriter(path);
			writer.WriteLine(size);
			for (int i = 0; i < size; i++)
			{
				//write vertex number
				writer.Write(i);

				//get and sort neighbours
				List<int> list = getNeighbours(i).ToList();
				list.Sort();

				//write them
				foreach (int j in list)
					writer.Write(" "+j);
				if(i<size-1)
					writer.WriteLine();
			}
			writer.Close();
		}

		public int degree(int v)
		{
			return degreeTable[v];
		}

		public abstract HashSet<int> getNeighbours(int v);
		public abstract void addEgde(int u, int v);
		public abstract void removeEdge(int u, int v);
		public abstract bool isEdge(int u, int v);
		
		public int Count
		{
			get
			{
				return size;
			}
		}

		public int NumEdges
		{
			get
			{
				if (degreeTableChanged)
				{

					numEdges = 0;
					for (int v = 0; v < size; v++)
					{
						numEdges += degree(v);
					}
					numEdges = (numEdges / 2);
					degreeTableChanged = false;
				}
				return numEdges;
			}
		}

		public override string ToString()
		{
			string name = this.GetType().Name;
			return name.Substring(name.LastIndexOf('.')+1) +", |V|: " + Count + ", |E|: " + NumEdges;
		}
	}


	
}
