﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	public class UndirectedGraphMatrix : UndirectedGraph
	{
		public BinaryMatrix matrix;

		public UndirectedGraphMatrix(int size) : base(size){}
		public UndirectedGraphMatrix(string path) : base(path) { }
		public UndirectedGraphMatrix(BinaryMatrix matrix) : base(matrix.M)
		{
			size = matrix.M;
			this.matrix = matrix;
		}
		protected override void init(int size)
		{
			matrix = new BinaryMatrixBitArrays(size, size);
		}

		override public void addEgde(int u, int v)
		{
			if (!isEdge(u, v))
			{
				matrix.set(u, v, true);
				matrix.set(v, u, true);
				degreeTable[u]++;
				degreeTable[v]++;
				degreeTableChanged = true;
			}
		}

		override public void removeEdge(int u, int v)
		{
			if (isEdge(u, v))
			{
				matrix.set(u, v, false);
				matrix.set(v, u, false);
				degreeTable[u]--;
				degreeTable[v]--;
				degreeTableChanged = true;
			}
		}

		override public bool isEdge(int u, int v)
		{
			return matrix.get(u, v);
		}

		override public HashSet<int> getNeighbours(int v)
		{
			HashSet<int> neighbours = new HashSet<int>(matrix.getIndexesByValuePerRow(v,true));
			return neighbours;
		}

	}
}
