﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;

namespace CommunityDetection
{
	class FilterCanidatesFile : ExpProgram
	{
		//queue of all candidates to be processed
		ThreadSafeQueue<Community> candidates;

		//generator
		CandidatesGenerator generator;

		double minDist;


		public FilterCanidatesFile()
		{

		}

		public override void loadArguments(string[] args)
		{
			generator = new CandidateFileGenerator(datadir+args[0]);
			minDist = double.Parse(args[1]);
		}

		public override void start()
		{
			Community.measure = new CommunityGain(false);
			CandidatesGenerator.smartStart = true;
			Operation.addWithoutGain = true;

			//candidates
			Console.Write("Generating candidates with " + generator + "... ");
			candidates = generator.generate(graph, null);
			int numCandidates = candidates.Count;
			Console.WriteLine("Done..");

			//Printing numbers:
			Console.WriteLine("Numbers:");
			Console.WriteLine(" - Graph: " + graph);
			Console.WriteLine(" - Candidates: " + candidates.Count);
				
		
			//sort results
			DateTime start = DateTime.Now;
			Console.Write("Filtering resultset of "+candidates.Count+" communities... ");
			StreamWriter writer = new StreamWriter(expdir+getExpName()+".cf");
			List<Community> communities = new List<Community>();
			while(!candidates.Empty())
			{
				Community c = candidates.Dequeue();
				bool similar = false;
				foreach (Community c2 in communities)
				{
					if (c.JaccardDistance(c2) < minDist)
					{
						similar = true;
						break;
					}
				}
				if(!similar)
					communities.Add(c);
			}
			TimeSpan durationSpan = DateTime.Now - start;
			int duration = (durationSpan.Days * 24 * 3600 + durationSpan.Hours * 3600 + durationSpan.Minutes * 60 + durationSpan.Seconds);
			Console.WriteLine("Done in " + duration + " seconds!");

			log("Datadir: " + datadir);
			log("Expdir: " + expdir);
			log("Graph: " + graphName);
			log("Candidates: " + numCandidates);
			log("Result communities: " + communities.Count);
			log("Duration: " + duration);
			log("Minimum Jaccard distance: " + minDist);


			Console.Write("Sorting communities on communitygain and size.. ");
			communities = new List<Community>(communities.OrderByDescending(c => c.communityGain()).ThenBy(c => c.Size));
			Console.WriteLine("Done!");

			Console.Write("Writing result file of " + communities.Count + " communities... ");
			foreach(Community c in communities)
			{
				List<int> nodes = new List<int>(c.getVertices().Keys);
				nodes.Sort();
				string line = "";
				foreach (int v in nodes)
				{
					line+= " "+v;
				}
				writer.WriteLine(line.Trim());
			}
			writer.Close();
			Console.WriteLine("Done!");
		}

	}
}
