﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.Globalization;

namespace CommunityDetection
{
	public class TestEMClassifier : ExpProgram
	{
		public CandidateFileGenerator generator;
		BinaryMatrix matrix;
		string datafile;
		public StreamWriter jobwriter;
		HashSet<int> trueRows;

		public override void loadArguments(string[] args)
		{
			generator = new CandidateFileGenerator(datadir+args[0]);
			datafile = datadir+args[1];
		}

		public override void start()
		{
			jobwriter = new StreamWriter(expdir + expName + ".csv");
			String s = ";";
			String line = "Maximum pattern length" + s + "alpha";
			line += s + "Duration (s)" + s + "TP" + s + "FP" + s + "TN" + s + "FN";
			line += s + "Precision" + s + "Recall";
			explog(line);
			Console.Write("Reading description data.. ");
			matrix = new BinaryMatrixBitArrays(datafile);
			Console.WriteLine("Done!");

			log("Description data: " + matrix.M + "x" + matrix.N);
			Console.Write("Creating community");
			ThreadSafeQueue<Community> queue = generator.generate(graph, matrix);
			Community c = queue.Dequeue();
			Console.WriteLine("Intra cluster density C:" + c.intraClusterDensity());
			log("Intra cluster density C:" + c.intraClusterDensity());
			log("CG(c): " + c.communityGain());
			Console.WriteLine("CG(c):" + c.communityGain());
			trueRows = new HashSet<int>(c.getVertices().Keys);
			Console.WriteLine("Done! Community: " + c.Size);
			log("Community size: " + c.Size);

			TimeMeasure measure = new TimeMeasure();
			EMBuilder builder = new EMBuilder(matrix);
			EMClassifier classifier = (EMClassifier) builder.build(new HashSet<int>(trueRows));
			double duration = measure.getDuration();
			int[] q = classifier.quality(matrix, trueRows);
			line += s + duration  + s + q[0] + s + q[1] + s + q[2] + s + q[3]+classifier.precision(q)+s+classifier.recall(q);
			Console.WriteLine("P: "+classifier.precision(q) + "\nR:" + classifier.recall(q));
			explog(line);
			log("Candidatesfile: " + datafile);

		}


	
	
		public void explog(string msg)
		{
			lock(jobwriter)
			{
				jobwriter.WriteLine(msg);
				jobwriter.Flush();
			}
		}
	}
}
