﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;

namespace CommunityDetection
{
	public class EMDM
	{
		//graph
		public UndirectedGraph graph;

		//matrix
		public BinaryMatrix description;

		//candidates
		CandidatesGenerator generator;

		//classifier
		Builder builder;

		//object to store the result
		ResultSet result;

		//list of all situations seen till now
		CommunityStampSet stamps;

		//queue of all candidates to be processed
		ThreadSafeQueue<Community> candidates;

		//directorys
		string datadir, expdir;

		//files
		string graphName, descriptionName;

		//number of threads to run program
		int numThreads;

		public void runExp()
		{
			if (new int[]{1,2,3}.Contains(0))
			{
				StreamReader config = new StreamReader("config.cfg");
				datadir = config.ReadLine();// "D:/Projecten/Afstuderen/svn/data/";
				expdir = config.ReadLine();// "D:/Projecten/Afstuderen/svn/xps/";

				graphName = config.ReadLine();// "delicious/graph";
				descriptionName = config.ReadLine();// "delicious/desc_small";
				if (descriptionName == "")
					descriptionName = null;
				numThreads = int.Parse(config.ReadLine()); //8

				//settings
				string gentype = config.ReadLine();
				if (gentype == "sn")
				{
					generator = new SharedNeighboursCG(int.Parse(config.ReadLine()));
				}
				if (gentype == "file")
				{
					generator = new CandidateFileGenerator(datadir + config.ReadLine());
				}
				config.Close();
			}
			else
			{
				datadir = "D:/Projecten/Afstuderen/svn/data/";
				expdir = "D:/Projecten/Afstuderen/svn/xps/";

				graphName = "delicious/graph.gph";
				descriptionName = "delicious/desc_small.dsm";
				if (descriptionName == "")
					descriptionName = null;
				numThreads = 1;
				generator = new SharedNeighboursCG(2);
			}
			Community.measure = new CommunityGain(false);
			CandidatesGenerator.smartStart = true;
			Operation.addWithoutGain = true;

			//read graph
			Console.Write("Reading graph... ");
			graph = new UndirectedGraphList(datadir + graphName);
			Console.WriteLine("Done!");

			//description data
			if (descriptionName != null)
			{
				Console.Write("Reading description matrix... ");
				description = new BinaryMatrixBitArrays(datadir + descriptionName);
				Console.WriteLine("Done!");

				if (description != null)
				{
					Console.Write("Preparing classifier... ");
					builder = new RemineBuilder(description);
					Console.WriteLine("Done!");
				}
			}


			//candidates
			Console.Write("Generating candidates with " + generator + "... ");
			candidates = generator.generate(graph, description);
			int numCandidates = candidates.Count;
			Console.WriteLine("Done..");

			//result set
			Console.Write("Creating result set... ");
			result = new ResultSet(graph);
			Console.WriteLine("Done!");

			//stamp set
			Console.Write("Creating stamp set... ");
			stamps = new CommunityStampSet();
			Console.WriteLine("Done!");

			//Printing numbers:
			Console.WriteLine("Numbers:");
			Console.WriteLine(" - Graph: " + graph);
			Console.WriteLine(" - Description data: " + (description == null ? "none" : description.ToString()));
			Console.WriteLine(" - Candidates: " + candidates.Count);
			Console.WriteLine(" - Threads: " + numThreads);

			//running emdm
			DateTime start = DateTime.Now;
			Console.WriteLine("Processing candidate" + (candidates.Count > 1 ? "s" : "") + " ... ");

			//creating threads
			List<Thread> threads = new List<Thread>();
			for (int i = 0; i < numThreads; i++)
			{
				Thread t = new Thread(this.run);
				threads.Add(t);
				t.Start();
			}

			//wait for all threads to be finished
			foreach (Thread t in threads)
				t.Join();

			//total duration
			TimeSpan durationSpan = DateTime.Now - start;
			int duration = (durationSpan.Days * 24 * 3600 + durationSpan.Hours * 3600 + durationSpan.Minutes * 60 + durationSpan.Seconds);
			Console.WriteLine("Done in " + duration + " seconds!");

			//sort results
			Console.Write("Sorting resultset of size " + result.Count + "... ");
			List<Community> list = result.toSortedList();
			Console.WriteLine("Done!");

			//write to file
			string expname = expdir + "run_" + DateTime.Now.ToString("yyyy_MM_dd_HHmmss");
			Directory.CreateDirectory(expname);
			StreamWriter sw = new StreamWriter(expname + ".csv");
			sw.WriteLine("Generator:;" + generator.ToString());
			sw.WriteLine("Smartstart:;" + (CandidatesGenerator.smartStart ? "Yes" : "No"));
			sw.WriteLine("Graph:;" + graph);
			Community all = new Community(graph);
			for (int i = 0; i < graph.Count; i++)
				all.AddVertice(i);
			sw.WriteLine(" ICD:;" + all.intraClusterDensity());
			sw.WriteLine("Description data:;" + (description == null ? "none" : description.ToString()));
			sw.WriteLine(" name:; " + descriptionName);
			sw.WriteLine("|Candidates|;" + numCandidates);
			sw.WriteLine("Add without gain:;" + (Operation.addWithoutGain ? "Yes" : "No"));
			sw.WriteLine("Duration (s):;" + duration);
			sw.WriteLine("Threads:;" + numThreads);
			sw.WriteLine("Resultset:;" + list.Count);
			sw.WriteLine("Top-k Jaccard distance:");

			//write something about top-k statistics
			sw.WriteLine("k;Average;Min;Max;Stdev;AVG|C|;AVG CG(C)");
			int[] sizes = new int[] { 2, 3, 4, 5, 6, 7, 8, 9, 10, 25, 100, 250 };
			foreach (int k in sizes)
			{
				if (k > result.Count)
					break;
				double[] jd = result.avgJaccardDistance(k);
				sw.WriteLine(k + ";" + jd[0] + ";" + jd[1] + ";" + jd[2] + ";" + jd[3] + ";" + jd[4] + ";" + jd[5]);
			}

			//write found communities sorted on CG(C) and Size(C)
			int id = 1;
			sw.WriteLine("id;|C|;CG(C);ICD(C);|T|;C;Steps");
			foreach (Community c in list)
			{
				List<int> vlist = new List<int>(c.getVertices().Keys);
				vlist.Sort();
				string vertices = "";
				foreach (int i in vlist)
					vertices += i + " ";
				sw.WriteLine(id + ";" + c.Size + ";" + c.communityGain() + ";" + c.intraClusterDensity() + ";" + c.treeSize + ";" + vertices + ";" + c.numOp);
				if (c.classifier != null)
					c.classifier.Write(expname + "/c_" + id + ".txt");
				id++;
			}
			sw.Close();
		}

		public void run()
		{
			try
			{
				while (!candidates.Empty())
				{
					Community c = candidates.Dequeue();
					if (stamps.Contains(c))
					{
						continue;
					}
					stamps.Add(c);
					if (emdm(c) != null)
						result.Add(c);
					if (candidates.Count % 25 == 0 && candidates.Count > 0)
						Console.WriteLine("  " + candidates.Count + " candidates left , stamps: " + stamps.Count);
				}
			}
			catch (InvalidOperationException)
			{
				//Console.WriteLine(e);
			}
		}

		public Community emdm(Community c)
		{
			while (true) //check later
			{
				if (!em(c))
					return null;
				if (!stamps.Contains(c))
					return null;
				dm(c);
				return c;
			}
		}

		public bool em(Community c)
		{
			while (c.improve())
			{
				//check if this situation is seen before
				if (stamps.Contains(c))
				{
					return false; //seen before, go on;
				}
				stamps.Add(c);
			}
			return true;
		}

		public void dm(Community c)
		{
			//check if there is description data
			if (description == null || this.builder == null)
				return;

			//build the classifier
			RemineClassifier classifier = (RemineClassifier)builder.build(new HashSet<int>(c.getVertices().Keys));

			//classify each row
			for (int i = 0; i < graph.Count; i++)
			{
				bool classified = classifier.classify(description.getRow(i));
				if (classified && !c.ContainsVertice(i))
				{
					c.AddVertice(i);
				}
				else if (!classified && c.ContainsVertice(i))
				{
					c.RemoveVertice(i);
				}
			}
			c.classifier = classifier;
			stamps.Add(c);
		}

	}
}
