﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using CommunityDetection;

namespace DSMConverter
{
    class Program
    {
        public Program(bool bToDsm, string input, string output, int format)
        {
            if (bToDsm)
            {
                toDsm(input, output, format);
            }
            else
            {
                fromDsm(input, output, format);
            }
 
        }
        public void toDsm(string input, string output, int format)
        {
            List<List<int>> list = new List<List<int>>();
            int max = 0;
            StreamReader reader = new StreamReader(input);
            string line = null;
            Console.Write("Reading transactions.. ");
            while (null != (line = reader.ReadLine()))
            {
                if (line.Length > 0 && (line[0] == '#' || line[0] == ';' || line[0] == '%'))
                    continue;
                //line = line.Replace(':', ' ').Replace("("," ").Replace(")"," ");
                line = string.Join("",line.Select(x => "0123456789".Contains(x) ? ""+x : " "));
                while (line.Contains("  "))
                    line = line.Replace("  "," ");
                line = line.Trim();
                List<int> att = new List<int>(line.Split(' ').Select(x => int.Parse(x)));
                if (format <= att.Count)
                {
                    att = new List<int>(att.Skip(format));
                }

                //Console.WriteLine(Tools.listToString(att));
                if(att.Count > 0)
                    max = Math.Max(max, att.Max());
                list.Add(att);
            }
            reader.Close();
            Console.WriteLine("Done!");

            
            Console.Write("Creating matrix.. ");
            BinaryMatrixBitArrays dm = new BinaryMatrixBitArrays(list.Count, max+1);
            Console.WriteLine("Done!");

            Console.Write("Setting matrix values.. ");
            for(int i=0; i<list.Count; i++)
            {
                foreach (int j in list[i])
                {
                    dm.set(i, j, true);
                }
            }
            Console.WriteLine("Done!");

            Console.Write("Writing to dsm file.. ");
            dm.Write(output);
            Console.WriteLine("Done!");


        }
        public void fromDsm(string input, string output, int format)
        {
            Console.Write("Reading matrix.. ");
            BinaryMatrix dm = new BinaryMatrixBitArrays(input);
            Console.WriteLine("Done!");

            if ((format & 16) > 0)
            {
                dm.WriteArffFile(output,"dataset");
            }
            else
            {
                StreamWriter writer = new StreamWriter(output);
                if ((format & 8) > 0)
                {
                    Console.Write("Calculating properties.. ");
                    writer.WriteLine("# transactions: " + dm.M);
                    writer.WriteLine("# attributes: " + dm.N);
                    writer.WriteLine(("# density: " + dm.Density).Replace(",","."));
                    Console.WriteLine("Done!");
                }
                Console.Write("Writing transactions.. ");
                for (int i = 0; i < dm.M; i++)
                {
                    List<int> list = dm.getIndexesByValuePerRow(i, true);
                    list.Sort();
                    if ((format & 1) > 0)
                    {
                        writer.Write(i);
                    }
                    if ((format & 2) > 0)
                    {
                        writer.Write(" ("+list.Count+")");
                    }
                    if ((format & 4) > 0)
                    {
                        writer.Write(":");
                    }
                    if ((format & 1) + (format & 2)+ (format & 4) > 0 && list.Count > 0)
                        writer.Write(" ");

                    writer.WriteLine(string.Join(" ", list.Select(x => x + "")));
                }
                writer.Close();
            }
            Console.WriteLine("Done!");
        }

        static void Main(string[] args)
        {
            Console.WriteLine("** Convert from/to DSM files **");
            if(args.Length < 3)
            {
                Console.WriteLine("Error: Invalid number of parameters (>=3), press key to exit");
                Console.ReadKey();
                return;
            }
            try
            {
                new Program(args[0].ToLower() == "todsm", args[1], args[2], args.Length==4 ? int.Parse(args[3]):0);
            }
            catch (Exception e)
            {
               Console.WriteLine("Error: " + e.Message);
            }
            Console.WriteLine("Press key to exit");
            Console.ReadKey();
        }
    }
}
